@@ -1,6 +1,8 @@
-/* $Id: CoinFactorization1.cpp 1277 2010-04-21 20:19:32Z forrest $ */
+/* $Id: CoinFactorization1.cpp 1736 2014-08-24 21:27:47Z tkr $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
 #if defined(_MSC_VER)
 // Turn off compiler warning about long names
 #  pragma warning(disable:4786)
@@ -13,6 +15,7 @@
 #include "CoinIndexedVector.hpp"
 #include "CoinHelperFunctions.hpp"
 #include "CoinPackedMatrix.hpp"
+#include "CoinFinite.hpp"
 #include <stdio.h>
 //:class CoinFactorization.  Deals with Factorization and Updates
 //  CoinFactorization.  Constructor
@@ -31,11 +34,47 @@ CoinFactorization::CoinFactorization ( const CoinFactorization &other)
   gutsOfCopy(other);
 }
 /// The real work of constructors etc
-void CoinFactorization::gutsOfDestructor(int )
+/// Really really delete if type 2
+void CoinFactorization::gutsOfDestructor(int type)
 {
   delete [] denseArea_;
   delete [] densePermute_;
-
+  if (type==2) {
+    elementU_.switchOff();
+    startRowU_.switchOff();
+    convertRowToColumnU_.switchOff();
+    indexRowU_.switchOff();
+    indexColumnU_.switchOff();
+    startColumnU_.switchOff();
+    elementL_.switchOff();
+    indexRowL_.switchOff();
+    startColumnL_.switchOff();
+    startColumnR_.switchOff();
+    numberInRow_.switchOff();
+    numberInColumn_.switchOff();
+    numberInColumnPlus_.switchOff();
+    pivotColumn_.switchOff();
+    pivotColumnBack_.switchOff();
+    firstCount_.switchOff();
+    nextCount_.switchOff();
+    lastCount_.switchOff();
+    permute_.switchOff();
+    permuteBack_.switchOff();
+    nextColumn_.switchOff();
+    lastColumn_.switchOff();
+    nextRow_.switchOff();
+    lastRow_.switchOff();
+    saveColumn_.switchOff();
+    markRow_.switchOff();
+    pivotRowL_.switchOff();
+    pivotRegion_.switchOff();
+    elementByRowL_.switchOff();
+    startRowL_.switchOff();
+    indexColumnL_.switchOff();
+    sparse_.switchOff();
+    workArea_.switchOff();
+    workArea2_.switchOff();
+  }
   elementU_.conditionalDelete();
   startRowU_.conditionalDelete();
   convertRowToColumnU_.conditionalDelete();
@@ -252,6 +291,8 @@ int CoinFactorization::factorize (
   const int * columnLength = matrix.getVectorLengths(); 
   const double * element = matrix.getElements();
   int numberRows=matrix.getNumRows();
+  if (!numberRows)
+    return 0;
   int numberColumns=matrix.getNumCols();
   int numberBasic = 0;
   CoinBigIndex numberElements=0;
@@ -486,7 +527,7 @@ CoinFactorization::factorizePart2 (int permutation[],int exactNumberElements)
 //  ~CoinFactorization.  Destructor
 CoinFactorization::~CoinFactorization (  )
 {
-  gutsOfDestructor();
+  gutsOfDestructor(2);
 }
 
 //  show_self.  Debug show object
@@ -966,7 +1007,7 @@ CoinFactorization::factor (  )
 	int * nextRow = nextRow_.array();
 	//int nSing =0;
 	k=nextRow[maximumRowsExtra_];
-	while (k!=maximumRowsExtra_) {
+	while (k!=maximumRowsExtra_ && k>=0) {
 	  int iRow = k;
 	  k=nextRow[k];
 	  //nSing++;
@@ -2382,5 +2423,5 @@ CoinFactorization::setPersistenceFlag(int flag)
 void 
 CoinFactorization::almostDestructor()
 {
-  gutsOfDestructor(1);
+  gutsOfDestructor(2);
 }
