@@ -1,5 +1,7 @@
+/* $Id: CoinFactorization2.cpp 1416 2011-04-17 09:57:29Z stefan $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
 #if defined(_MSC_VER)
 // Turn off compiler warning about long names
@@ -14,6 +16,7 @@
 #include "CoinFactorization.hpp"
 #include "CoinIndexedVector.hpp"
 #include "CoinHelperFunctions.hpp"
+#include "CoinFinite.hpp"
 #if DENSE_CODE==1
 // using simple lapack interface
 extern "C" 
@@ -356,7 +359,7 @@ CoinFactorization::factorSparseSmall (  )
 	    //branch out to best pivot routine 
 	    goodPivot = pivot ( iPivotRow, iPivotColumn,
 				pivotRowPosition, pivotColumnPosition,
-				workArea, workArea2, increment,
+				workArea, workArea2, 
 				increment2,  markRow ,
 				SMALL_SET);
 #else
@@ -836,7 +839,7 @@ CoinFactorization::restoreFactorization (const char * file , bool factorIt )
   if (fp) {
     // Get rid of current
     gutsOfDestructor();
-    int newSize=0; // for checking - should be same
+    CoinBigIndex newSize=0; // for checking - should be same
     // Restore so we can pick up scalars
     char * first = reinterpret_cast<char *> ( &pivotTolerance_);
     char * last = reinterpret_cast<char *> ( &biasLU_);
@@ -1294,7 +1297,7 @@ CoinFactorization::factorSparseLarge (  )
 	      //branch out to best pivot routine 
 	      goodPivot = pivot ( iPivotRow, iPivotColumn,
 				  pivotRowPosition, pivotColumnPosition,
-				  workArea, workArea2, increment,
+				  workArea, workArea2, 
 				  increment2, markRow ,
 				  LARGE_SET);
 #else
