@@ -1,4 +1,4 @@
-/* $Id: CoinLpIO.cpp 1272 2010-04-06 00:18:00Z ladanyi $ */
+/* $Id: CoinLpIO.cpp 1728 2014-08-05 16:33:45Z tkr $ */
 // Last edit: 11/5/08
 //
 // Name:     CoinLpIO.cpp; Support for Lp files
@@ -9,7 +9,11 @@
 //-----------------------------------------------------------------------------
 // Copyright (C) 2003, Francois Margot, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
+#include "CoinUtilsConfig.h"
+
+#include <cmath>
 #include <cfloat>
 #include <cctype>
 #include <cassert>
@@ -20,6 +24,7 @@
 #include "CoinHelperFunctions.hpp"
 #include "CoinPackedMatrix.hpp"
 #include "CoinLpIO.hpp"
+#include "CoinFinite.hpp"
 
 using namespace std;
 
@@ -29,6 +34,7 @@ using namespace std;
 
 CoinLpIO::CoinLpIO() :
   problemName_(CoinStrdup("")),
+  defaultHandler_(true),
   numberRows_(0),
   numberColumns_(0),
   numberElements_(0),
@@ -41,16 +47,19 @@ CoinLpIO::CoinLpIO() :
   rhs_(NULL),
   rowrange_(NULL),
   rowsense_(NULL),
-  objective_(NULL),
-  objectiveOffset_(0),
+  num_objectives_(0),
   integerType_(NULL),
   fileName_(NULL),
-  infinity_(DBL_MAX),
+  infinity_(COIN_DBL_MAX),
   epsilon_(1e-5),
   numberAcross_(10),
-  decimals_(5),
-  objName_(NULL)
+  decimals_(5)
 {
+  for (int j = 0; j < MAX_OBJECTIVES; j++){
+     objective_[j] = NULL;
+     objName_[j] = NULL;
+     objectiveOffset_[j] = 0;
+  }
   card_previous_names_[0] = 0;
   card_previous_names_[1] = 0;
   previous_names_[0] = NULL;
@@ -64,13 +73,192 @@ CoinLpIO::CoinLpIO() :
   numberHash_[1] = 0;
   hash_[1] = NULL;
   names_[1] = NULL;
+  handler_ = new CoinMessageHandler();
+  messages_ = CoinMessage();
 }
 
+//-------------------------------------------------------------------
+// Copy constructor
+//-------------------------------------------------------------------
+CoinLpIO::CoinLpIO(const CoinLpIO& rhs)
+    :
+    problemName_(CoinStrdup("")),
+    defaultHandler_(true),
+    numberRows_(0),
+    numberColumns_(0),
+    numberElements_(0),
+    matrixByColumn_(NULL),
+    matrixByRow_(NULL),
+    rowlower_(NULL),
+    rowupper_(NULL),
+    collower_(NULL),
+    colupper_(NULL),
+    rhs_(NULL),
+    rowrange_(NULL),
+    rowsense_(NULL),
+    integerType_(NULL),
+    fileName_(CoinStrdup("")),
+    infinity_(COIN_DBL_MAX),
+    epsilon_(1e-5),
+    numberAcross_(10),
+    decimals_(5)
+{
+    for (int j = 0; j < MAX_OBJECTIVES; j++){
+       objective_[j] = NULL;
+       objName_[j] = NULL;
+       objectiveOffset_[j] = 0;
+    }
+    card_previous_names_[0] = 0;
+    card_previous_names_[1] = 0;
+    previous_names_[0] = NULL;
+    previous_names_[1] = NULL;
+    maxHash_[0] = 0;
+    numberHash_[0] = 0;
+    hash_[0] = NULL;
+    names_[0] = NULL;
+    maxHash_[1] = 0;
+    numberHash_[1] = 0;
+    hash_[1] = NULL;
+    names_[1] = NULL;
+ 
+    if ( rhs.rowlower_ != NULL || rhs.collower_ != NULL) {
+       gutsOfCopy(rhs);
+    }
+ 
+    defaultHandler_ = rhs.defaultHandler_;
+ 
+    if (defaultHandler_) {
+       handler_ = new CoinMessageHandler(*rhs.handler_);
+    } else {
+       handler_ = rhs.handler_;
+    }
+ 
+    messages_ = CoinMessage();
+}
+
+
+void CoinLpIO::gutsOfCopy(const CoinLpIO& rhs)
+{
+    defaultHandler_ = rhs.defaultHandler_;
+ 
+    if (rhs.matrixByRow_) {
+       matrixByRow_ = new CoinPackedMatrix(*(rhs.matrixByRow_));
+    }
+ 
+    numberElements_ = rhs.numberElements_;
+    numberRows_ = rhs.numberRows_;
+    numberColumns_ = rhs.numberColumns_;
+    decimals_ = rhs.decimals_;
+    num_objectives_ = rhs.num_objectives_;
+ 
+    if (rhs.rowlower_) {
+       rowlower_ = reinterpret_cast<double*> (malloc(numberRows_ * sizeof(double)));
+       rowupper_ = reinterpret_cast<double*> (malloc(numberRows_ * sizeof(double)));
+       memcpy(rowlower_, rhs.rowlower_, numberRows_ * sizeof(double));
+       memcpy(rowupper_, rhs.rowupper_, numberRows_ * sizeof(double));
+       rowrange_ = reinterpret_cast<double *> (malloc(numberRows_*sizeof(double)));
+       rowsense_ = reinterpret_cast<char *> (malloc(numberRows_*sizeof(char)));
+       rhs_ = reinterpret_cast<double *> (malloc(numberRows_*sizeof(double)));
+       memcpy(rowrange_,rhs.getRowRange(),numberRows_*sizeof(double));
+       memcpy(rowsense_,rhs.getRowSense(),numberRows_*sizeof(char));
+       memcpy(rhs_,rhs.getRightHandSide(),numberRows_*sizeof(double));
+    }
+ 
+    if (rhs.collower_) {
+       collower_ = reinterpret_cast<double*> (malloc(numberColumns_ * sizeof(double)));
+       colupper_ = reinterpret_cast<double*> (malloc(numberColumns_ * sizeof(double)));
+       memcpy(collower_, rhs.collower_, numberColumns_ * sizeof(double));
+       memcpy(colupper_, rhs.colupper_, numberColumns_ * sizeof(double));
+       for (int j = 0; j < num_objectives_; j++){
+	  objective_[j] = reinterpret_cast<double*> (malloc(numberColumns_ * sizeof(double)));
+          memcpy(objective_[j], rhs.objective_[j], numberColumns_ * sizeof(double));
+       }
+    }
+ 
+    if (rhs.integerType_) {
+       integerType_ = reinterpret_cast<char*> (malloc (numberColumns_ * sizeof(char)));
+       memcpy(integerType_, rhs.integerType_, numberColumns_ * sizeof(char));
+    }
+ 
+    free(fileName_);
+    free(problemName_);
+    fileName_ = CoinStrdup(rhs.fileName_);
+    problemName_ = CoinStrdup(rhs.problemName_);
+    numberHash_[0] = rhs.numberHash_[0];
+    numberHash_[1] = rhs.numberHash_[1];
+    maxHash_[0] = rhs.maxHash_[0];
+    maxHash_[1] = rhs.maxHash_[1];
+    infinity_ = rhs.infinity_;
+    numberAcross_ = rhs.numberAcross_;
+    for (int j = 0; j < num_objectives_; j++){
+       objectiveOffset_[j] = rhs.objectiveOffset_[j];
+    }
+    int section;
+ 
+    for (section = 0; section < 2; section++) {
+       if (numberHash_[section]) {
+          char** names2 = rhs.names_[section];
+          names_[section] = reinterpret_cast<char**> (malloc(maxHash_[section] *
+                            sizeof(char*)));
+          char** names = names_[section];
+          int i;
+ 
+          for (i = 0; i < numberHash_[section]; i++) {
+             names[i] = CoinStrdup(names2[i]);
+          }
+ 
+          hash_[section] = new CoinHashLink[maxHash_[section]];
+          std::memcpy(hash_[section], rhs.hash_[section], maxHash_[section]*sizeof(CoinHashLink));
+       }
+    }
+ }
+
+CoinLpIO &
+CoinLpIO::operator=(const CoinLpIO& rhs)
+{
+    if (this != &rhs) {
+       gutsOfDestructor();
+ 
+       if ( rhs.rowlower_ != NULL || rhs.collower_ != NULL) {
+          gutsOfCopy(rhs);
+       }
+ 
+       defaultHandler_ = rhs.defaultHandler_;
+ 
+       if (defaultHandler_) {
+          handler_ = new CoinMessageHandler(*rhs.handler_);
+       } else {
+          handler_ = rhs.handler_;
+       }
+ 
+       messages_ = CoinMessage();
+    }
+ 
+    return *this;
+}
+
+
+void CoinLpIO::gutsOfDestructor()
+ {
+    freeAll();
+ 
+    if (defaultHandler_) {
+       delete handler_;
+       handler_ = NULL;
+    }
+}
+
+
+
 /************************************************************************/
 CoinLpIO::~CoinLpIO() {
   stopHash(0);
   stopHash(1);
   freeAll();
+  if (defaultHandler_) {
+    delete handler_;
+    handler_ = NULL; 
+  }
 }
 
 /************************************************************************/
@@ -94,7 +282,9 @@ void
 CoinLpIO::freeAll() {
 
   delete matrixByColumn_;
+  matrixByColumn_ = NULL; 
   delete matrixByRow_;
+  matrixByRow_ = NULL; 
   free(rowupper_);
   rowupper_ = NULL;
   free(rowlower_);
@@ -109,8 +299,10 @@ CoinLpIO::freeAll() {
   rowrange_ = NULL;
   free(rowsense_);
   rowsense_ = NULL;
-  free(objective_);
-  objective_ = NULL;
+  for (int j = 0; j < num_objectives_; j++){
+     free(objective_[j]);
+     objective_[j] = NULL;
+  }
   free(integerType_);
   integerType_ = NULL;
   free(problemName_);
@@ -261,10 +453,22 @@ const double * CoinLpIO::getRowRange() const
   return rowrange_;
 }
 
+/*************************************************************************/
+const int CoinLpIO::getNumObjectives() const
+{
+  return num_objectives_;
+}
+ 
 /*************************************************************************/
 const double * CoinLpIO::getObjCoefficients() const
 {
-  return objective_;
+  return objective_[0];
+}
+ 
+/*************************************************************************/
+const double * CoinLpIO::getObjCoefficients(int j) const
+{
+  return objective_[j];
 }
  
 /*************************************************************************/
@@ -286,7 +490,13 @@ const CoinPackedMatrix * CoinLpIO::getMatrixByCol() const
 /*************************************************************************/
 const char * CoinLpIO::getObjName() const
 {
-  return objName_;
+  return objName_[0];
+}
+ 
+/*************************************************************************/
+const char * CoinLpIO::getObjName(int j) const
+{
+  return objName_[j];
 }
  
 /*************************************************************************/
@@ -296,7 +506,9 @@ void CoinLpIO::checkRowNames() {
 
   if(numberHash_[0] != nrow+1) {
     setDefaultRowNames();
-    printf("### WARNING: CoinLpIO::checkRowNames(): non distinct or missing row names or objective function name.\nNow using default row names.\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::checkRowNames(): non distinct or missing row names or objective function name.\nNow using default row names."
+						     <<CoinMessageEol;
   }
 
   char const * const * rowNames = getRowNames();
@@ -311,7 +523,10 @@ void CoinLpIO::checkRowNames() {
       sprintf(rName, "%s_low", rowNames[i]);
       if(findHash(rName, 0) != -1) {
 	setDefaultRowNames();
-	printf("### WARNING: CoinLpIO::checkRowNames(): ranged constraint %d hasa name %s identical to another constraint name or objective function name.\nUse getPreviousNames() to get the old row names.\nNow using default row names.\n", i, rName);
+	char printBuffer[512];
+	sprintf(printBuffer,"### CoinLpIO::checkRowNames(): ranged constraint %d has a name %s identical to another constraint name or objective function name.\nUse getPreviousNames() to get the old row names.\nNow using default row names.", i, rName);
+	handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						  <<CoinMessageEol;
 	break;
       }
     }
@@ -325,7 +540,9 @@ void CoinLpIO::checkColNames() {
 
   if(numberHash_[1] != ncol) {
     setDefaultColNames();
-    printf("### WARNING: CoinLpIO::checkColNames(): non distinct or missing column names.\nNow using default column names.\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::checkColNames(): non distinct or missing column names.\nNow using default column names."
+					      <<CoinMessageEol;
   }
 } /* checkColNames */
 
@@ -476,7 +693,13 @@ void CoinLpIO::setDecimals(const int value)
 /************************************************************************/
 double CoinLpIO::objectiveOffset() const
 {
-  return objectiveOffset_;
+  return objectiveOffset_[0];
+}
+
+/************************************************************************/
+double CoinLpIO::objectiveOffset(int j) const
+{
+  return objectiveOffset_[j];
 }
 
 /************************************************************************/
@@ -504,6 +727,20 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
 			      const char *is_integer,
 			      const double *rowlb, const double *rowub) {
 
+   setLpDataWithoutRowAndColNames(m, collb, colub, &obj_coeff, 1, is_integer, 
+				  rowlb, rowub);
+}
+
+/************************************************************************/
+void
+CoinLpIO::setLpDataWithoutRowAndColNames(
+			      const CoinPackedMatrix& m,
+			      const double *collb, const double *colub,
+			      const double *obj_coeff[MAX_OBJECTIVES],
+			      int num_objectives,
+			      const char *is_integer,
+			      const double *rowlb, const double *rowub) {
+
   freeAll();
   problemName_ = CoinStrdup("");
 
@@ -521,12 +758,15 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
   rowupper_ = reinterpret_cast<double *> (malloc (numberRows_ * sizeof(double)));
   collower_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
   colupper_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
-  objective_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
   std::copy(rowlb, rowlb + numberRows_, rowlower_);
   std::copy(rowub, rowub + numberRows_, rowupper_);
   std::copy(collb, collb + numberColumns_, collower_);
   std::copy(colub, colub + numberColumns_, colupper_);
-  std::copy(obj_coeff, obj_coeff + numberColumns_, objective_);
+  num_objectives_ = num_objectives;
+  for (int j = 0; j < num_objectives; j++){
+     objective_[j] = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
+     std::copy(obj_coeff[j], obj_coeff[j] + numberColumns_, objective_[j]);
+  }
 
   if (is_integer) {
     integerType_ = reinterpret_cast<char *> (malloc (numberColumns_ * sizeof(char)));
@@ -560,7 +800,7 @@ void CoinLpIO::setDefaultRowNames() {
 
   stopHash(0);
   startHash(defaultRowNames, nrow+1, 0);
-  objName_ = CoinStrdup("obj");
+  objName_[0] = CoinStrdup("obj");
 
   for(i=0; i<nrow+1; i++) {
     free(defaultRowNames[i]);
@@ -600,25 +840,29 @@ void CoinLpIO::setLpDataRowAndColNames(char const * const * const rownames,
   if(rownames != NULL) {
     if(are_invalid_names(rownames, nrow+1, true)) {
       setDefaultRowNames();
-      printf("### WARNING: CoinLpIO::setLpDataRowAndColNames(): Invalid row names\nUse getPreviousNames() to get the old row names.\nNow using default row names.\n");
+      handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::setLpDataRowAndColNames(): Invalid row names\nUse getPreviousNames() to get the old row names.\nNow using default row names."
+						<<CoinMessageEol;
     } 
     else {
       stopHash(0);
       startHash(rownames, nrow+1, 0);
-      objName_ = CoinStrdup(rownames[nrow]);
+      objName_[0] = CoinStrdup(rownames[nrow]);
       checkRowNames();
     }
   }
   else {
-    if(objName_ == NULL) {
-      objName_ = CoinStrdup("obj");      
+    if(objName_[0] == NULL) {
+      objName_[0] = CoinStrdup("obj");      
     }
   }
 
   if(colnames != NULL) {
     if(are_invalid_names(colnames, ncol, false)) {
       setDefaultColNames();
-      printf("### WARNING: CoinLpIO::setLpDataRowAndColNames(): Invalid column names\nNow using default row names.\n");
+      handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::setLpDataRowAndColNames(): Invalid column names\nNow using default row names."
+						<<CoinMessageEol;
     } 
     else {
       stopHash(1);
@@ -727,7 +971,6 @@ CoinLpIO::writeLp(FILE *fp, const bool useRowNames)
    const double *colup = getColUpper();
    const double *rowlow = getRowLower();
    const double *rowup = getRowUpper();
-   const double *obj = getObjCoefficients();
    const char *integerType = integerColumns();
    char const * const * rowNames = getRowNames();
    char const * const * colNames = getColNames();
@@ -765,37 +1008,39 @@ CoinLpIO::writeLp(FILE *fp, const bool useRowNames)
    fprintf(fp, "\\Problem name: %s\n\n", getProblemName());
    fprintf(fp, "Minimize\n");
 
-   if(useRowNames) {
-     fprintf(fp, "%s:", objName_);
-   }
+   for (int k = 0; k < num_objectives_; k++){
+      if(useRowNames) {
+	 fprintf(fp, "%s:", objName_[k]);
+      }
 
-   cnt_print = 0;
-   for(j=0; j<ncol; j++) {
-     if((cnt_print > 0) && (objective_[j] > lp_eps)) {
-       fprintf(fp, " +");
-     }
-     if(fabs(obj[j]) > lp_eps) {
-       out_coeff(fp, obj[j], 0);
-       fprintf(fp, " %s", colNames[j]);
-       cnt_print++;
-       if(cnt_print % numberAcross == 0) {
-	 fprintf(fp, "\n");
-       }
-     }
-   }
+      cnt_print = 0;
+      for(j=0; j<ncol; j++) {
+	 if((cnt_print > 0) && (objective_[k][j] > lp_eps)) {
+	    fprintf(fp, " +");
+	 }
+	 if(fabs(objective_[k][j]) > lp_eps) {
+	    out_coeff(fp, objective_[k][j], 0);
+	    fprintf(fp, " %s", colNames[j]);
+	    cnt_print++;
+	    if(cnt_print % numberAcross == 0) {
+	       fprintf(fp, "\n");
+	    }
+	 }
+      }
  
-   if((cnt_print > 0) && (objectiveOffset_ > lp_eps)) {
-     fprintf(fp, " +");
-   }
-   if(fabs(objectiveOffset_) > lp_eps) {
-     out_coeff(fp, objectiveOffset_, 1);
-     cnt_print++;
+      if((cnt_print > 0) && (objectiveOffset_[k] > lp_eps)) {
+	 fprintf(fp, " +");
+      }
+      if(fabs(objectiveOffset_[k]) > lp_eps) {
+	 out_coeff(fp, objectiveOffset_[k], 1);
+	 cnt_print++;
+      }
+      if((cnt_print == 0) || (cnt_print % numberAcross != 0)) {
+	 fprintf(fp, "\n");
+      }
+   
    }
 
-   if(cnt_print % numberAcross != 0) {
-     fprintf(fp, "\n");
-   }
-   
    fprintf(fp, "Subject To\n");
    
    int cnt_out_rows = 0;
@@ -1130,21 +1375,32 @@ CoinLpIO::is_invalid_name(const char *name,
     lname = strlen(name);
   }
   if(lname < 1) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name is empty\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+    "### CoinLpIO::is_invalid_name(): Name is empty"
+					      <<CoinMessageEol;
     return(5);
   }
   if(lname > valid_lname) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name %s is too long\n", 
+	char printBuffer[512];
+	sprintf(printBuffer,"### CoinLpIO::is_invalid_name(): Name %s is too long", 
 	   name);
+	handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							 <<CoinMessageEol;
     return(1);
   }
   if(first_is_number(name)) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name %s should not start with a number\n", name);
+    char printBuffer[512];
+    sprintf(printBuffer,"### CoinLpIO::is_invalid_name(): Name %s should not start with a number", name);
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						     <<CoinMessageEol;
     return(2);
   }
   pos = strspn(name, str_valid);
   if(pos != lname) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name %s contains illegal character '%c'\n", name, name[pos]);
+    char printBuffer[512];
+    sprintf(printBuffer,"### CoinLpIO::is_invalid_name(): Name %s contains illegal character '%c'", name, name[pos]);
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						     <<CoinMessageEol;
     return(3);
   }
 
@@ -1182,8 +1438,11 @@ CoinLpIO::are_invalid_names(char const * const * const vnames,
     }
     flag = is_invalid_name(vnames[i], is_ranged);
     if(flag) {
-      printf("### WARNING: CoinLpIO::are_invalid_names(): Invalid name: vnames[%d]: %s\n",
-	     i, vnames[i]);
+      char printBuffer[512];
+      sprintf(printBuffer,"### CoinLpIO::are_invalid_names(): Invalid name: vnames[%d]: %s",
+	      i, vnames[i]);
+      handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						       <<CoinMessageEol;
       invalid = flag;
     }
   }
@@ -1193,7 +1452,7 @@ CoinLpIO::are_invalid_names(char const * const * const vnames,
 /*************************************************************************/
 int 
 CoinLpIO::read_monom_obj(FILE *fp, double *coeff, char **name, int *cnt, 
-						 char **obj_name) {
+			 char **obj_name, int *num_objectives, int *obj_starts) {
 
   double mult;
   char buff[1024] = "aa", loc_name[1024], *start;
@@ -1214,7 +1473,14 @@ CoinLpIO::read_monom_obj(FILE *fp, double *coeff, char **name, int *cnt,
     printf("CoinLpIO: read_monom_obj(): obj_name: %s\n", buff);
 #endif
 
-    *obj_name = CoinStrdup(buff);
+    if (*num_objectives == MAX_OBJECTIVES){
+       char str[8192];
+       sprintf(str,"### ERROR: Too many objective functions.\n");
+       sprintf(str,"### ERROR: Change MAX_OBJECTIVES to larger number.\n");
+       throw CoinError(str, "read_monom_obj", "CoinLpIO", __FILE__, __LINE__);
+    }
+    obj_name[*num_objectives] = CoinStrdup(buff);
+    obj_starts[(*num_objectives)++] = *cnt;
     return(0);
   }
 
@@ -1254,7 +1520,7 @@ CoinLpIO::read_monom_obj(FILE *fp, double *coeff, char **name, int *cnt,
   }
   
   if(first_is_number(start)) {
-    coeff[*cnt] = atof(start);       
+     coeff[*cnt] = atof(start);       
     sprintf(loc_name, "aa");
     scan_next(loc_name, fp);
   }
@@ -1341,7 +1607,14 @@ CoinLpIO::read_monom_row(FILE *fp, char *start_str,
   }
 
   coeff[cnt_coeff] *= mult;
+#ifdef KILL_ZERO_READLP
+  if (fabs(coeff[cnt_coeff])>epsilon_)
+    name[cnt_coeff] = CoinStrdup(loc_name);
+  else
+    read_sense=-2; // effectively zero
+#else
   name[cnt_coeff] = CoinStrdup(loc_name);
+#endif
 
 #ifdef LPIO_DEBUG
   printf("CoinLpIO: read_monom_row: (%f)  (%s)\n", 
@@ -1385,6 +1658,13 @@ CoinLpIO::realloc_col(double **collow, double **colup, char **is_int,
   *collow = reinterpret_cast<double *> (realloc ((*collow), (*maxcol+1) * sizeof(double)));
   *colup = reinterpret_cast<double *> (realloc ((*colup), (*maxcol+1) * sizeof(double)));
   *is_int = reinterpret_cast<char *> (realloc ((*is_int), (*maxcol+1) * sizeof(char)));
+  // clean values
+  double lp_inf = getInfinity();
+  for (int i=(*maxcol)-100; i<*maxcol; i++) {
+    (*collow)[i] = 0;
+    (*colup)[i] = lp_inf;
+    (*is_int)[i] = 0;
+  }
 
 } /* realloc_col */
 
@@ -1409,8 +1689,10 @@ CoinLpIO::read_row(FILE *fp, char *buff,
     }
     read_sense = read_monom_row(fp, start_str, 
 				*pcoeff, *pcolNames, *cnt_coeff);
-
-    (*cnt_coeff)++;
+#ifdef KILL_ZERO_READLP
+    if (read_sense!=-2) // see if zero
+#endif
+      (*cnt_coeff)++;
 
     scan_next(start_str, fp);
 
@@ -1514,11 +1796,11 @@ CoinLpIO::readLp(FILE* fp)
   char buff[1024];
 
   int objsense, cnt_coeff = 0, cnt_row = 0, cnt_obj = 0;
-  char *objName = NULL;
-
+  int num_objectives = 0;
+  char *objName[MAX_OBJECTIVES];
+  int obj_starts[MAX_OBJECTIVES+1];
   char **colNames = reinterpret_cast<char **> (malloc ((maxcoeff+1) * sizeof(char *)));
   double *coeff = reinterpret_cast<double *> (malloc ((maxcoeff+1) * sizeof(double)));
-
   char **rowNames = reinterpret_cast<char **> (malloc ((maxrow+1) * sizeof(char *)));
   int *start = reinterpret_cast<int *> (malloc ((maxrow+1) * sizeof(int)));
   double *rhs = reinterpret_cast<double *> (malloc ((maxrow+1) * sizeof(double)));
@@ -1531,13 +1813,14 @@ CoinLpIO::readLp(FILE* fp)
 
   int read_st = 0;
   while(!read_st) {
-    read_st = read_monom_obj(fp, coeff, colNames, &cnt_obj, &objName);
+     read_st = read_monom_obj(fp, coeff, colNames, &cnt_obj, objName, &num_objectives, obj_starts);
 
     if(cnt_obj == maxcoeff) {
       realloc_coeff(&coeff, &colNames, &maxcoeff);
     }
   }
   
+  obj_starts[num_objectives] = cnt_obj;
   start[0] = cnt_obj;
   cnt_coeff = cnt_obj;
 
@@ -1656,7 +1939,10 @@ CoinLpIO::readLp(FILE* fp)
 
 	icol = findHash(buff, 1);
 	if(icol < 0) {
-	  printf("### WARNING: CoinLpIO::readLp(): Variable %s does not appear in objective function or constraints\n", buff);
+	  char printBuffer[512];
+	  sprintf(printBuffer,"### CoinLpIO::readLp(): Variable %s does not appear in objective function or constraints", buff);
+	  handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							   <<CoinMessageEol;
 	  insertHash(buff, 1);
 	  icol = findHash(buff, 1);
 	  if(icol == maxcol) {
@@ -1766,7 +2052,10 @@ CoinLpIO::readLp(FILE* fp)
 #endif
 
 	if(icol < 0) {
-	  printf("### WARNING: CoinLpIO::readLp(): Integer variable %s does not appear in objective function or constraints\n", buff);
+	  char printBuffer[512];
+	  sprintf(printBuffer,"### CoinLpIO::readLp(): Integer variable %s does not appear in objective function or constraints", buff);
+	  handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							   <<CoinMessageEol;
 	  insertHash(buff, 1);
 	  icol = findHash(buff, 1);
 	  if(icol == maxcol) {
@@ -1799,7 +2088,10 @@ CoinLpIO::readLp(FILE* fp)
 #endif
 
 	if(icol < 0) {
-	  printf("### WARNING: CoinLpIO::readLp(): Binary variable %s does not appear in objective function or constraints\n", buff);
+	  char printBuffer[512];
+	  sprintf(printBuffer,"### CoinLpIO::readLp(): Binary variable %s does not appear in objective function or constraints", buff);
+	  handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							   <<CoinMessageEol;
 	  insertHash(buff, 1);
 	  icol = findHash(buff, 1);
 	  if(icol == maxcol) {
@@ -1858,22 +2150,30 @@ CoinLpIO::readLp(FILE* fp)
   numberColumns_ = numberHash_[1];
   numberElements_ = cnt_coeff - start[0];
 
-  double *obj = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
-  memset(obj, 0, numberColumns_ * sizeof(double));
+  double *obj[MAX_OBJECTIVES];
 
-  for(i=0; i<cnt_obj; i++) {
-    icol = findHash(colNames[i], 1);
-    if(icol < 0) {
-      char str[8192];
-      sprintf(str,"### ERROR: Hash table: %s (obj) not found\n", colNames[i]);
-      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
-    }
-    obj[icol] = objsense * coeff[i];
+  for (int j = 0; j < num_objectives; j++){
+     obj[j] = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
+     memset(obj[j], 0, numberColumns_ * sizeof(double));
+
+     for(i=obj_starts[j]; i<obj_starts[j+1]; i++) {
+       icol = findHash(colNames[i], 1);
+       if(icol < 0) {
+         char str[8192];
+         sprintf(str,"### ERROR: Hash table: %s (obj) not found\n", colNames[i]);
+         throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
+       }
+       obj[j][icol] = objsense * coeff[i];
+     }
   }
 
   if (objsense == -1) {
-    printf("### WARNING: CoinLpIO::readLp(): Maximization problem reformulated as minimization\n");
-    objectiveOffset_ = -objectiveOffset_;
+    handler_->message(COIN_GENERAL_INFO,messages_)<<
+      " CoinLpIO::readLp(): Maximization problem reformulated as minimization"
+						  <<CoinMessageEol;
+    for (int j = 0; j < num_objectives_; j++){
+       objectiveOffset_[j] = -objectiveOffset_[j];
+    }
   }
 
 
@@ -1891,22 +2191,26 @@ CoinLpIO::readLp(FILE* fp)
 #endif
 
   setLpDataWithoutRowAndColNames(*matrix, collow, colup,
-				 obj, has_int ? is_int : 0, rowlow, rowup);
+				 const_cast<const double **>(obj), 
+				 num_objectives, has_int ? is_int : 0, rowlow, rowup);
 
 
-  if(objName == NULL) {
-    rowNames[cnt_row] = CoinStrdup("obj");
+  for (int j = 0; j < num_objectives; j++){
+     if(objName[j] == NULL) {
+	rowNames[cnt_row+j] = CoinStrdup("obj");
+     }
+     else {
+	rowNames[cnt_row+j] = CoinStrdup(objName[j]);
+     }
   }
-  else {
-    rowNames[cnt_row] = CoinStrdup(objName);
-  }
-
   // Hash tables for column names are already set up
   setLpDataRowAndColNames(rowNames, NULL);
 
   if(are_invalid_names(names_[1], numberHash_[1], false)) {
     setDefaultColNames();
-    printf("### WARNING: CoinLpIO::readLp(): Invalid column names\nNow using default column names.\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::readLp(): Invalid column names\nNow using default column names."
+						     <<CoinMessageEol;
   } 
   
   for(i=0; i<cnt_coeff; i++) {
@@ -1919,7 +2223,9 @@ CoinLpIO::readLp(FILE* fp)
   }
   free(rowNames);
 
-  free(objName);
+  for (int j = 0; j < num_objectives; j++){
+     free(objName[j]);
+  }
 
 #ifdef LPIO_DEBUG
   writeLp("readlp.xxx");
@@ -1935,10 +2241,12 @@ CoinLpIO::readLp(FILE* fp)
   free(rowlow);
   free(rowup);
   free(is_int);
-  free(obj);
+  for (int j = 0; j < num_objectives; j++){
+    free(obj[j]);
+  }
   delete matrix;
 
-} /* readLp */
+ } /* readLp */
 
 /*************************************************************************/
 void
@@ -1976,9 +2284,11 @@ CoinLpIO::print() const {
   }
   printf("\n");
   
-  printf("objective_:\n");
-  for(i=0; i<numberColumns_; i++) {
-    printf("%.5f ", objective_[i]);
+  for (int j = 0; j < num_objectives_; j++){
+     printf("objective_[%i]:\n", j);
+     for(i=0; i<numberColumns_; i++) {
+	printf("%.5f ", objective_[j][i]);
+     }
   }
   printf("\n");
   
@@ -2003,10 +2313,8 @@ CoinLpIO::print() const {
 /*************************************************************************/
 // Hash functions slightly modified from CoinMpsIO.cpp
 
-static int
-compute_hash(const char *name, int maxsiz, int length)
-{
-  static int mmult[] = {
+namespace {
+  const int mmult[] = {
     262139, 259459, 256889, 254291, 251701, 249133, 246709, 244247,
     241667, 239179, 236609, 233983, 231289, 228859, 226357, 223829,
     221281, 218849, 216319, 213721, 211093, 208673, 206263, 203773,
@@ -2018,6 +2326,9 @@ compute_hash(const char *name, int maxsiz, int length)
     103387, 101021, 98639, 96179, 93911, 91583, 89317, 86939, 84521,
     82183, 79939, 77587, 75307, 72959, 70793, 68447, 66103
   };
+ int compute_hash(const char *name, int maxsiz, int length)
+{
+  
   int n = 0;
   int j;
 
@@ -2028,6 +2339,7 @@ compute_hash(const char *name, int maxsiz, int length)
   }
   return ( abs ( n ) % maxsiz );	/* integer abs */
 }
+} // end file-local namespace
 
 /************************************************************************/
 //  startHash.  Creates hash list for names
@@ -2161,8 +2473,6 @@ CoinLpIO::startHash(char const * const * const names,
 void
 CoinLpIO::stopHash(int section)
 {
-  char **names = names_[section];
-
   freePreviousNames(section);
   previous_names_[section] = names_[section];
   card_previous_names_[section] = numberHash_[section];
@@ -2170,14 +2480,14 @@ CoinLpIO::stopHash(int section)
   delete[] hash_[section];
   hash_[section] = NULL;
 
-  names = NULL;
-
   maxHash_[section] = 0;
   numberHash_[section] = 0;
 
   if(section == 0) {
-    free(objName_);
-    objName_ = NULL;
+     for (int j = 0; j < num_objectives_; j++){
+	free(objName_[j]);
+	objName_[j] = NULL;
+     }
   }
 } /* stopHash */
 
@@ -2285,4 +2595,19 @@ CoinLpIO::insertHash(const char *thisName, int section)
   (numberHash_[section])++;
 
 }
+// Pass in Message handler (not deleted at end)
+void 
+CoinLpIO::passInMessageHandler(CoinMessageHandler * handler)
+{
+  if (defaultHandler_)
+    delete handler_;
+  defaultHandler_=false;
+  handler_=handler;
+}
+// Set language
+void 
+CoinLpIO::newLanguage(CoinMessages::Language language)
+{
+  messages_ = CoinMessage(language);
+}
 
