@@ -1,3 +1,4 @@
+/* $Id: CoinLpIO.hpp 1727 2014-08-05 16:30:20Z tkr $ */
 // Last edit: 11/5/08
 //
 // Name:     CoinLpIO.hpp; Support for Lp files
@@ -9,10 +10,17 @@
 //-----------------------------------------------------------------------------
 // Copyright (C) 2003, Francois Margot, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
+#ifndef CoinLpIO_H
+#define CoinLpIO_H
 
 #include <cstdio>
 
-class CoinPackedMatrix;
+#include "CoinPackedMatrix.hpp"
+#include "CoinMessage.hpp"
+
+const int MAX_OBJECTIVES = 2;
 
 typedef int COINColumnIndex;
 
@@ -68,7 +76,10 @@ Notes: <UL>
       as suffix. This should be kept in mind when assigning names to ranged
       constraint, as the resulting name must be distinct from all the other
       names and be considered valid by the method is_invalid_name().
- <LI> At most one constant term may appear in the objective function; 
+ <LI> At most one term related to any single variable may appear in the
+      objective function; if more than one term are present, only the last
+      one is taken into account.
+      At most one constant term may appear in the objective function; 
       if present, it must appear last. 
  <LI> Default bounds are 0 for lower bound and +infinity for upper bound.
  <LI> Free variables get default lower bound -infinity and 
@@ -88,7 +99,7 @@ Notes: <UL>
 </UL>
 */
 class CoinLpIO {
-
+      friend void CoinLpIOUnitTest(const std::string & lpDir); 
 public:
 
   /**@name Constructor and Destructor */
@@ -96,6 +107,18 @@ public:
   /// Default Constructor
   CoinLpIO(); 
   
+  /// Does the heavy lifting for destruct and assignment.
+  void gutsOfDestructor(); 
+ 
+  /// Does the heavy lifting for copy and assignment
+  void gutsOfCopy(const CoinLpIO &); 
+ 
+  /// assignment operator
+  CoinLpIO & operator = (const CoinLpIO& rhs) ; 
+ 
+  /// Copy constructor 
+  CoinLpIO (const CoinLpIO &); 
+
   /// Destructor 
   ~CoinLpIO();
 
@@ -184,9 +207,15 @@ public:
   */
   const double * getRowRange() const;
 
+  /// Get pointer to array[getNumCols()] of objective function coefficients
+  const int getNumObjectives() const;
+  
   /// Get pointer to array[getNumCols()] of objective function coefficients
   const double * getObjCoefficients() const;
   
+  /// Get pointer to array[getNumCols()] of objective function coefficients for objective j
+  const double * getObjCoefficients(int j) const;
+  
   /// Get pointer to row-wise copy of the coefficient matrix
   const CoinPackedMatrix * getMatrixByRow() const;
 
@@ -196,6 +225,9 @@ public:
   /// Get objective function name
   const char * getObjName() const;
   
+  /// Get objective function name for objective j
+  const char * getObjName(int j) const;
+  
   /// Get pointer to array[*card_prev] of previous row names.
   /// The value of *card_prev might be different than getNumRows()+1 if 
   /// non distinct
@@ -240,9 +272,16 @@ public:
   ///Returns the (constant) objective offset
   double objectiveOffset() const;
   
+  ///Returns the (constant) objective offset for objective j
+  double objectiveOffset(int j) const;
+  
   /// Set objective offset
   inline void setObjectiveOffset(double value)
-  { objectiveOffset_ = value;}
+  { objectiveOffset_[0] = value;}
+  
+  /// Set objective offset
+   inline void setObjectiveOffset(double value, int j)
+  { objectiveOffset_[j] = value;}
   
   /// Return true if a column is an integer (binary or general 
   /// integer) variable
@@ -292,12 +331,21 @@ public:
       The sense of optimization of the objective function is assumed to be 
       a minimization. 
       Numbers larger than DBL_MAX (or larger than 1e+400) 
-      might crash the code.
+      might crash the code. There are two version. The second one is for
+      setting multiple objectives.
   */
   void setLpDataWithoutRowAndColNames(
 			      const CoinPackedMatrix& m,
 			      const double* collb, const double* colub,
-			      const double* obj_coeff, 
+			      const double* obj_coeff,
+			      const char* integrality,
+			      const double* rowlb, const double* rowub);
+
+  void setLpDataWithoutRowAndColNames(
+			      const CoinPackedMatrix& m,
+			      const double* collb, const double* colub,
+			      const double* obj_coeff[MAX_OBJECTIVES],
+			      int num_objectives,
 			      const char* integrality,
 			      const double* rowlb, const double* rowub);
 
@@ -440,11 +488,45 @@ public:
   /// Dump the data. Low level method for debugging.
   void print() const;
   //@}
+/**@name Message handling */
+//@{
+  /** Pass in Message handler
+  
+      Supply a custom message handler. It will not be destroyed when the
+      CoinMpsIO object is destroyed.
+  */
+  void passInMessageHandler(CoinMessageHandler * handler);
+
+  /// Set the language for messages.
+  void newLanguage(CoinMessages::Language language);
+
+  /// Set the language for messages.
+  inline void setLanguage(CoinMessages::Language language) {newLanguage(language);}
+
+  /// Return the message handler
+  inline CoinMessageHandler * messageHandler() const {return handler_;}
+
+  /// Return the messages
+  inline CoinMessages messages() {return messages_;}
+  /// Return the messages pointer
+  inline CoinMessages * messagesPointer() {return & messages_;}
+//@}
 
 protected:
   /// Problem name
   char * problemName_;
 
+  /// Message handler
+  CoinMessageHandler * handler_;
+  /** Flag to say if the message handler is the default handler.
+      
+      If true, the handler will be destroyed when the CoinMpsIO
+      object is destroyed; if false, it will not be destroyed.
+  */
+  bool defaultHandler_;
+  /// Messages
+  CoinMessages messages_;
+
   /// Number of rows
   int numberRows_;
   
@@ -484,10 +566,13 @@ protected:
   mutable char * rowsense_;
   
   /// Pointer to dense vector of objective coefficients
-  double * objective_;
+  double * objective_[MAX_OBJECTIVES];
+
+  /// Number of objectives
+  int num_objectives_;
   
   /// Constant offset for objective value
-  double objectiveOffset_;
+  double objectiveOffset_[MAX_OBJECTIVES];
   
   /// Pointer to dense vector specifying if a variable is continuous
   /// (0) or integer (1).
@@ -509,7 +594,7 @@ protected:
   int decimals_;
 
   /// Objective function name
-  char *objName_;
+  char *objName_[MAX_OBJECTIVES];
 
   /** Row names (including objective function name) 
       and column names when stopHash() for the corresponding 
@@ -635,7 +720,7 @@ protected:
   /// Read a monomial of the objective function.
   /// Return 1 if "subject to" or one of its variants has been read.
   int read_monom_obj(FILE *fp, double *coeff, char **name, int *cnt, 
-		     char **obj_name);
+		     char **obj_name, int *num_objectives, int *obj_starts);
 
   /// Read a monomial of a constraint.
   /// Return a positive number if the sense of the inequality has been 
@@ -688,3 +773,8 @@ protected:
 
 };
 
+void
+CoinLpIOUnitTest(const std::string& lpDir);
+
+
+#endif 
