@@ -1,10 +1,15 @@
+/* $Id: CoinModel.hpp 1726 2014-08-05 16:15:35Z tkr $ */
 // Copyright (C) 2005, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
 #ifndef CoinModel_H
 #define CoinModel_H
 
 #include "CoinModelUseful.hpp"
+#include "CoinMessageHandler.hpp"
 #include "CoinPackedMatrix.hpp"
+#include "CoinFinite.hpp"
 class CoinBaseModel {
 
 public:
@@ -77,6 +82,8 @@ public:
   /// Set column block name
   inline void setColumnBlock(const std::string &name) 
   { columnBlockName_ = name;}
+  /// Pass in message handler
+  void setMessageHandler(CoinMessageHandler * handler);
    //@}
   
 protected:
@@ -96,9 +103,15 @@ protected:
   std::string rowBlockName_;
   /// Columnblock name
   std::string columnBlockName_;
+  /// Message handler (Passed in)
+  CoinMessageHandler * handler_;
+  /// Messages 
+  CoinMessages messages_;
+  
   /** Print level.
       I could have gone for full message handling but this should normally
-      be silent and lightweight.  I can always change.
+      be silent and lightweight.
+      -1 - use passed in message handler
       0 - no output
       1 - on errors
       2 - more detailed
@@ -691,6 +704,12 @@ public:
   /// Return column names array
   inline const CoinModelHash * columnNames() const
   { return &columnName_;}
+  /// Reset row names
+  inline void zapRowNames()
+  { rowName_=CoinModelHash();}
+  /// Reset column names
+  inline void zapColumnNames()
+  { columnName_=CoinModelHash();}
   /// Returns array of 0 or nonzero if can be a cut (or returns NULL)
   inline const int * cutMarker() const
   { return cut_;}
@@ -806,13 +825,15 @@ public:
    //@{
    /** Default constructor. */
    CoinModel();
-    /** Read a problem in MPS or GAMS format from the given filename.
-    */
-    CoinModel(const char *fileName, int allowStrings=0);
-    /** Read a problem from AMPL nl file
-	NOTE - as I can't work out configure etc the source code is in Cbc_ampl.cpp!
-    */
-    CoinModel( int nonLinear, const char * fileName,const void * info);
+   /** Constructor with sizes. */
+   CoinModel(int firstRows, int firstColumns, int firstElements,bool noNames=false);
+   /** Read a problem in MPS or GAMS format from the given filename.
+   */
+   CoinModel(const char *fileName, int allowStrings=0);
+   /** Read a problem from AMPL nl file
+       NOTE - as I can't work out configure etc the source code is in Cbc_ampl.cpp!
+   */
+   CoinModel( int nonLinear, const char * fileName,const void * info);
   /// From arrays
   CoinModel(int numberRows, int numberColumns,
 	    const CoinPackedMatrix * matrix,
@@ -1015,6 +1036,8 @@ private:
       3 matrix is CoinPackedMatrix (and at present can't be modified);
   */
   mutable int type_;
+  /// True if no names EVER being used (for users who know what they are doing)
+  bool noNames_;
   /** Links present (could be tested by sizes of objects)
       0 - none,
       1 - row links,
