@@ -1,5 +1,7 @@
+/* $Id: CoinModelUseful.cpp 1585 2013-04-06 20:42:02Z stefan $ */
 // Copyright (C) 2005, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
 #include "CoinPragma.hpp"
 
@@ -66,6 +68,23 @@ CoinModelLink::operator=(const CoinModelLink& rhs)
   }
   return *this;
 }
+
+namespace {
+ const int mmult[] = {
+    262139, 259459, 256889, 254291, 251701, 249133, 246709, 244247,
+    241667, 239179, 236609, 233983, 231289, 228859, 226357, 223829,
+    221281, 218849, 216319, 213721, 211093, 208673, 206263, 203773,
+    201233, 198637, 196159, 193603, 191161, 188701, 186149, 183761,
+    181303, 178873, 176389, 173897, 171469, 169049, 166471, 163871,
+    161387, 158941, 156437, 153949, 151531, 149159, 146749, 144299,
+    141709, 139369, 136889, 134591, 132169, 129641, 127343, 124853,
+    122477, 120163, 117757, 115361, 112979, 110567, 108179, 105727,
+    103387, 101021, 98639, 96179, 93911, 91583, 89317, 86939, 84521,
+    82183, 79939, 77587, 75307, 72959, 70793, 68447, 66103
+  };
+  const int lengthMult = static_cast<int> (sizeof(mmult) / sizeof(int));
+}
+
 //#############################################################################
 // Constructors / Destructor / Assignment
 //#############################################################################
@@ -399,19 +418,7 @@ CoinModelHash::setName(int which,char * name )
 int 
 CoinModelHash::hashValue(const char * name) const
 {
-  static int mmult[] = {
-    262139, 259459, 256889, 254291, 251701, 249133, 246709, 244247,
-    241667, 239179, 236609, 233983, 231289, 228859, 226357, 223829,
-    221281, 218849, 216319, 213721, 211093, 208673, 206263, 203773,
-    201233, 198637, 196159, 193603, 191161, 188701, 186149, 183761,
-    181303, 178873, 176389, 173897, 171469, 169049, 166471, 163871,
-    161387, 158941, 156437, 153949, 151531, 149159, 146749, 144299,
-    141709, 139369, 136889, 134591, 132169, 129641, 127343, 124853,
-    122477, 120163, 117757, 115361, 112979, 110567, 108179, 105727,
-    103387, 101021, 98639, 96179, 93911, 91583, 89317, 86939, 84521,
-    82183, 79939, 77587, 75307, 72959, 70793, 68447, 66103
-  };
-  static int lengthMult = static_cast<int> (sizeof(mmult) / sizeof(int));
+  
   int n = 0;
   int j;
   int length =  static_cast<int> (strlen(name));
@@ -516,7 +523,7 @@ CoinModelHash2::resize(int maxItems, const CoinModelTriple * triples,bool forceR
    * collide with it are not entered.
    */
   for ( i = 0; i < numberItems_; ++i ) {
-    int row = static_cast<int> (triples[i].row);
+    int row = static_cast<int> (rowInTriple(triples[i]));
     int column = triples[i].column;
     if (column>=0) {
       ipos = hashValue ( row, column);
@@ -534,7 +541,7 @@ CoinModelHash2::resize(int maxItems, const CoinModelTriple * triples,bool forceR
    */
   lastSlot_ = -1;
   for ( i = 0; i < numberItems_; ++i ) {
-    int row = static_cast<int> (triples[i].row);
+    int row = static_cast<int> (rowInTriple(triples[i]));
     int column = triples[i].column;
     if (column>=0) {
       ipos = hashValue ( row, column);
@@ -545,7 +552,7 @@ CoinModelHash2::resize(int maxItems, const CoinModelTriple * triples,bool forceR
         if ( j1 == i )
           break;
         else {
-          int row2 = static_cast<int> (triples[j1].row);
+          int row2 = static_cast<int> (rowInTriple(triples[j1]));
           int column2 = triples[j1].column;
           if ( row==row2&&column==column2 ) {
             printf ( "** duplicate entry %d %d\n", row,column );
@@ -596,7 +603,7 @@ CoinModelHash2::hash(int row, int column, const CoinModelTriple * triples) const
     int j1 = hash_[ipos].index;
 
     if ( j1 >= 0 ) {
-      int row2 = static_cast<int> (triples[j1].row);
+      int row2 = static_cast<int> (rowInTriple(triples[j1]));
       int column2 = triples[j1].column;
       if ( row!=row2||column!=column2 ) {
 	int k = hash_[ipos].next;
@@ -640,7 +647,7 @@ CoinModelHash2::addHash(int index, int row, int column, const CoinModelTriple *
 	break; // duplicate??
       } else {
         if (j1 >=0 ) {
-          int row2 = static_cast<int> (triples[j1].row);
+          int row2 = static_cast<int> (rowInTriple(triples[j1]));
           int column2 = triples[j1].column;
           if ( row==row2&&column==column2 ) {
             printf ( "** duplicate entry %d %d\n", row, column );
@@ -696,14 +703,17 @@ CoinModelHash2::deleteHash(int index,int row, int column)
     }
   }
 }
+namespace {
+  const int mmult2[] = {
+    262139, 259459, 256889, 254291, 251701, 249133, 246709, 244247,
+    241667, 239179, 236609, 233983, 231289, 228859, 226357, 223829
+  };
+}
 // Returns a hash value
 int 
 CoinModelHash2::hashValue(int row, int column) const
 {
-  static int mmult[] = {
-    262139, 259459, 256889, 254291, 251701, 249133, 246709, 244247,
-    241667, 239179, 236609, 233983, 231289, 228859, 226357, 223829
-  };
+  
   // Optimizer should take out one side of if
   if (sizeof(int)==4*sizeof(char)) {
     unsigned char tempChar[4];
@@ -711,15 +721,15 @@ CoinModelHash2::hashValue(int row, int column) const
     unsigned int n = 0;
     int * temp = reinterpret_cast<int *> (tempChar);
     *temp=row;
-    n += mmult[0] * tempChar[0];
-    n += mmult[1] * tempChar[1];
-    n += mmult[2] * tempChar[2];
+    n += mmult2[0] * tempChar[0];
+    n += mmult2[1] * tempChar[1];
+    n += mmult2[2] * tempChar[2];
     n += mmult[3] * tempChar[3];
     *temp=column;
-    n += mmult[0+8] * tempChar[0];
-    n += mmult[1+8] * tempChar[1];
-    n += mmult[2+8] * tempChar[2];
-    n += mmult[3+8] * tempChar[3];
+    n += mmult2[0+8] * tempChar[0];
+    n += mmult2[1+8] * tempChar[1];
+    n += mmult2[2+8] * tempChar[2];
+    n += mmult2[3+8] * tempChar[3];
     return n % (maximumItems_<<1);
   } else {
     // ints are 8
@@ -731,12 +741,12 @@ CoinModelHash2::hashValue(int row, int column) const
     *temp=row;
     for ( j = 0; j < sizeof(int); ++j ) {
       int itemp = tempChar[j];
-      n += mmult[j] * itemp;
+      n += mmult2[j] * itemp;
     }
     *temp=column;
     for ( j = 0; j < sizeof(int); ++j ) {
       int itemp = tempChar[j];
-      n += mmult[j+8] * itemp;
+      n += mmult2[j+8] * itemp;
     }
     int maxHash = 4 * maximumItems_;
     int absN = abs(n);
@@ -888,7 +898,7 @@ CoinModelLinkedList::resize(int maxMajor,int maxElements)
 // Create list - for row list maxMajor is maximum rows
 void 
 CoinModelLinkedList::create(int maxMajor,int maxElements,
-                            int numberMajor,int numberMinor, int type,
+                            int numberMajor,int /*numberMinor*/, int type,
                             int numberElements, const CoinModelTriple * triples)
 {
   maxMajor=CoinMax(maxMajor,maximumMajor_);
@@ -925,13 +935,10 @@ CoinModelLinkedList::create(int maxMajor,int maxElements,
   for (i=0;i<numberElements;i++) {
     if (triples[i].column>=0) {
       int iMajor;
-      int iMinor;
       if (!type_) {
         // for rows
-        iMajor=static_cast<int> (triples[i].row);
-        iMinor=triples[i].column;
+        iMajor=static_cast<int> (rowInTriple(triples[i]));
       } else {
-        iMinor=static_cast<int> (triples[i].row);
         iMajor=triples[i].column;
       }
       assert (iMajor<numberMajor);
@@ -1007,18 +1014,16 @@ CoinModelLinkedList::addEasy(int majorIndex, int numberOfElements, const int * i
       }
       if (type_==0) {
         // row
-        triples[put].row=static_cast<unsigned int>(majorIndex);
-        triples[put].string=0;
+	setRowAndStringInTriple(triples[put],majorIndex,false);
         triples[put].column=indices[i];
       } else {
         // column
-        triples[put].row=static_cast<unsigned int>(indices[i]);
-        triples[put].string=0;
+	setRowAndStringInTriple(triples[put],indices[i],false);
         triples[put].column=majorIndex;
       }
       triples[put].value=elements[i];
       if (doHash)
-        hash.addHash(put,static_cast<int> (triples[put].row),triples[put].column,triples);
+        hash.addHash(put,static_cast<int> (rowInTriple(triples[put])),triples[put].column,triples);
       if (last>=0) {
         next_[last]=put;
       } else {
@@ -1068,18 +1073,16 @@ CoinModelLinkedList::addHard(int minorIndex, int numberOfElements, const int * i
     int other=indices[i];
     if (type_==0) {
       // row
-      triples[put].row=static_cast<unsigned int>(other);
-      triples[put].string=0;
+      setRowAndStringInTriple(triples[put],other,false);
       triples[put].column=minorIndex;
     } else {
       // column
-      triples[put].row=static_cast<unsigned int>(minorIndex);
-      triples[put].string=0;
+      setRowAndStringInTriple(triples[put],minorIndex,false);
       triples[put].column=other;
     }
     triples[put].value=elements[i];
     if (doHash)
-      hash.addHash(put,static_cast<int> (triples[put].row),triples[put].column,triples);
+      hash.addHash(put,static_cast<int> (rowInTriple(triples[put])),triples[put].column,triples);
     if (other>=numberMajor_) {
       // Need to fill in null values
       fill(numberMajor_,other+1);
@@ -1120,7 +1123,7 @@ CoinModelLinkedList::addHard(int first, const CoinModelTriple * triples,
     int other;
     if (type_==0) {
       // row
-      other=triples[put].row;
+      other=rowInTriple(triples[put]);
       if (minorIndex>=0)
         assert(triples[put].column==minorIndex);
       else
@@ -1129,9 +1132,9 @@ CoinModelLinkedList::addHard(int first, const CoinModelTriple * triples,
       // column
       other=triples[put].column;
       if (minorIndex>=0)
-        assert(static_cast<int> (triples[put].row)==minorIndex);
+        assert(static_cast<int> (rowInTriple(triples[put]))==minorIndex);
       else
-        minorIndex=triples[put].row;
+        minorIndex=rowInTriple(triples[put]);
     }
     assert (other<maximumMajor_);
     if (other>=numberMajor_) {
@@ -1165,7 +1168,7 @@ CoinModelLinkedList::deleteSame(int which, CoinModelTriple * triples,
     while (put>=0) {
       if (hash.numberItems()) {
         // take out of hash
-        hash.deleteHash(put, static_cast<int> (triples[put].row),triples[put].column);
+        hash.deleteHash(put, static_cast<int> (rowInTriple(triples[put])),triples[put].column);
       }
       if (zapTriples) {
         triples[put].column=-1;
@@ -1193,7 +1196,7 @@ CoinModelLinkedList::deleteSame(int which, CoinModelTriple * triples,
    This is when elements have been deleted from other copy
 */
 void 
-CoinModelLinkedList::updateDeleted(int which, CoinModelTriple * triples,
+CoinModelLinkedList::updateDeleted(int /*which*/, CoinModelTriple * triples,
                                    CoinModelLinkedList & otherList)
 {
   int firstFree = otherList.firstFree();
@@ -1213,7 +1216,7 @@ CoinModelLinkedList::updateDeleted(int which, CoinModelTriple * triples,
     int iMajor;
     if (!type_) {
       // for rows
-      iMajor=static_cast<int> (triples[lastFree].row);
+      iMajor=static_cast<int> (rowInTriple(triples[lastFree]));
     } else {
       iMajor=triples[lastFree].column;
     }
@@ -1223,29 +1226,33 @@ CoinModelLinkedList::updateDeleted(int which, CoinModelTriple * triples,
       int nextThis = next_[lastFree];
       if (previousThis>=0&&previousThis!=last) {
         next_[previousThis]=nextThis;
+#ifndef NDEBUG
         int iTest;
         if (!type_) {
           // for rows
-          iTest=static_cast<int> (triples[previousThis].row);
+          iTest=static_cast<int> (rowInTriple(triples[previousThis]));
         } else {
           iTest=triples[previousThis].column;
         }
         assert (triples[previousThis].column>=0);
         assert (iTest==iMajor);
+#endif
       } else {
         first_[iMajor]=nextThis;
       }
       if (nextThis>=0) {
         previous_[nextThis]=previousThis;
+#ifndef NDEBUG
         int iTest;
         if (!type_) {
           // for rows
-          iTest=static_cast<int> (triples[nextThis].row);
+          iTest=static_cast<int> (rowInTriple(triples[nextThis]));
         } else {
           iTest=triples[nextThis].column;
         }
         assert (triples[nextThis].column>=0);
         assert (iTest==iMajor);
+#endif
       } else {
         last_[iMajor]=previousThis;
       }
@@ -1259,7 +1266,7 @@ CoinModelLinkedList::updateDeleted(int which, CoinModelTriple * triples,
       if (previous>=0) {
         if (!type_) {
           // for rows
-          iMajor=static_cast<int> (triples[previous].row);
+          iMajor=static_cast<int> (rowInTriple(triples[previous]));
         } else {
           iMajor=triples[previous].column;
         }
@@ -1269,29 +1276,33 @@ CoinModelLinkedList::updateDeleted(int which, CoinModelTriple * triples,
           int nextThis = next_[previous];
           if (previousThis>=0&&previousThis!=last) {
             next_[previousThis]=nextThis;
+#ifndef NDEBUG
             int iTest;
             if (!type_) {
               // for rows
-              iTest=static_cast<int> (triples[previousThis].row);
+              iTest=static_cast<int> (rowInTriple(triples[previousThis]));
             } else {
               iTest=triples[previousThis].column;
             }
             assert (triples[previousThis].column>=0);
             assert (iTest==iMajor);
+#endif
           } else {
             first_[iMajor]=nextThis;
           }
           if (nextThis>=0) {
             previous_[nextThis]=previousThis;
+#ifndef NDEBUG
             int iTest;
             if (!type_) {
               // for rows
-              iTest=static_cast<int> (triples[nextThis].row);
+              iTest=static_cast<int> (rowInTriple(triples[nextThis]));
             } else {
               iTest=triples[nextThis].column;
             }
             assert (triples[nextThis].column>=0);
             assert (iTest==iMajor);
+#endif
           } else {
             last_[iMajor]=previousThis;
           }
@@ -1320,11 +1331,11 @@ void
 CoinModelLinkedList::deleteRowOne(int position, CoinModelTriple * triples,
                                   CoinModelHash2 & hash)
 {
-  int row=triples[position].row;
+  int row=rowInTriple(triples[position]);
   assert (row<numberMajor_);
   if (hash.numberItems()) {
     // take out of hash
-    hash.deleteHash(position, static_cast<int> (triples[position].row),triples[position].column);
+    hash.deleteHash(position, static_cast<int> (rowInTriple(triples[position])),triples[position].column);
   }
   int previous = previous_[position];
   int next = next_[position];
@@ -1419,25 +1430,19 @@ CoinModelLinkedList::validateLinks(const CoinModelTriple * triples) const
   int i;
   for ( i=0;i<numberMajor_;i++) {
     int position = first_[i];
+#ifndef NDEBUG
     int lastPosition=-1;
+#endif
     while (position>=0) {
-      int iMajor;
-      int iMinor;
-      if (position!=first_[i])
-        assert (next_[previous_[position]]==position);
-      if (!type_) {
-        // for rows
-        iMajor=static_cast<int> (triples[position].row);
-        iMinor=triples[position].column;
-      } else {
-        iMinor=static_cast<int> (triples[position].row);
-        iMajor=triples[position].column;
-      }
+      assert (position==first_[i] || next_[previous_[position]]==position);
+      assert (type_ || i == static_cast<int> (rowInTriple(triples[position])));  // i == iMajor for rows
+      assert (!type_ || i == triples[position].column);  // i == iMajor
       assert (triples[position].column>=0);
       mark[position]=1;
       lastElement = CoinMax(lastElement,position);
-      assert (i==iMajor);
+#ifndef NDEBUG
       lastPosition=position;
+#endif
       position = next_[position];
     }
     assert (lastPosition==last_[i]);
